@@ -575,6 +575,138 @@ void gtk_data_addfile(GtkWidget *btn, gpointer Adata)
   switch_pbar_status(Lcontenance, *Ldatamaxsize, *Ldatasize, PB_UPDATE_DATA);
 }
 
+/* l'utilisateur veut editer un device */
+void gtk_devices_edit(GtkWidget *Abtn, gpointer Adata)
+{
+  GHashTable *Lhash = (GHashTable *) Adata;
+  GtkWidget *Lparent = GTK_WIDGET(g_hash_table_lookup(Lhash, "window"));
+  GtkTreeView *Lliste = GTK_TREE_VIEW(g_hash_table_lookup(Lhash, "periphdata"));
+  GtkTreeSelection *Lselection = gtk_tree_view_get_selection(GTK_TREE_VIEW(Lliste));
+  GtkTreeIter Liter;
+  GtkTreeModel *Ltreemodel;
+
+  if (gtk_tree_selection_get_selected(Lselection, &Ltreemodel, &Liter)) {
+    GHashTable *Ldialoghash;
+    GtkWidget *Lwineditdevice;
+    Tdriveinfo *Ldrive;
+    gint Lnum;
+    gint Lbus;
+    
+    gtk_tree_model_get(Ltreemodel, &Liter, 0, &Lnum, -1);
+    Ldrive = (Tdriveinfo *)g_slist_nth_data(Glistdrives, Lnum);
+    Ldialoghash = create_dialog_edit_device(Lparent, Ldrive);
+    Lwineditdevice = g_hash_table_lookup(Ldialoghash, "window");
+
+    if (gtk_dialog_run(GTK_DIALOG(Lwineditdevice)) != GTK_RESPONSE_CANCEL) {
+      GtkEntry *Ldevicename = g_hash_table_lookup(Ldialoghash, "devicename");
+
+      Lbus = get_combo_intvalue(g_hash_table_lookup(Ldialoghash, "devicebus"));
+      Ldrive->actuperiph = Lbus;
+
+      /* on remplace le nom dans la liste des lecteurs */
+      g_free(Ldrive->name);
+      Ldrive->name = g_strdup(gtk_entry_get_text(Ldevicename));
+      /* ainsi que dans la liste affiche */
+      gtk_list_store_set(GTK_LIST_STORE(Ltreemodel), &Liter, 2, (gchar *)g_slist_nth_data(Ldrive->dev, Ldrive->actuperiph),
+          4, Ldrive->name, -1);
+    }
+    gtk_widget_destroy(Lwineditdevice);
+    g_hash_table_destroy(Ldialoghash);
+  }
+
+}
+
+/* l'utilisateur a cliquer sur supprimer un device */
+void gtk_devices_remove(GtkWidget *Abtn, gpointer Adata)
+{
+  GHashTable *Lhash = (GHashTable *)Adata;
+  GtkWidget *Lwindow1 = GTK_WIDGET(g_hash_table_lookup(Lhash, "topwindow"));
+  GtkWidget *Lconfirm;
+  gint Lrep;
+  
+  /* confirmation debut de l'operation */ 
+  Lconfirm = gtk_message_dialog_new(GTK_WINDOW(Lwindow1),
+                      GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION,
+                      GTK_BUTTONS_YES_NO,
+                      _("Do you really want to remove this device ?"));
+  Lrep = gtk_dialog_run(GTK_DIALOG(Lconfirm));
+  gtk_widget_destroy(Lconfirm);
+
+  if (Lrep == GTK_RESPONSE_YES) {
+    /* oui on supprime un device */
+    GtkTreeView *Lliste = GTK_TREE_VIEW(g_hash_table_lookup(Lhash, "periphdata"));
+    GtkTreeSelection *Lselection = gtk_tree_view_get_selection(GTK_TREE_VIEW(Lliste));
+    GtkTreeIter Liter;
+    GtkTreeModel *Ltreemodel;
+    gint Lnum;
+
+    if (gtk_tree_selection_get_selected(Lselection, &Ltreemodel, &Liter)) {
+      gtk_tree_model_get(Ltreemodel, &Liter, 0, &Lnum, -1);
+
+      _DEB("ON SUPIMME LE LEC = %d\n", Lnum);
+      Glistdrives = g_slist_remove(Glistdrives, g_slist_nth_data(Glistdrives, Lnum));
+      gtk_list_store_remove(GTK_LIST_STORE(Ltreemodel), &Liter);
+    } 
+  }
+
+}
+
+/* ajouter manuelement un device */
+void gtk_devices_add(GtkWidget *Abtn, gpointer Adata)
+{
+  GHashTable *Lhash = (GHashTable *) Adata;
+  GtkWidget *Lparent = GTK_WIDGET(g_hash_table_lookup(Lhash, "window"));
+  GtkTreeView *Lliste = GTK_TREE_VIEW(g_hash_table_lookup(Lhash, "periphdata"));
+  GHashTable *Ldialoghash = create_dialog_add_device(Lparent);
+  GtkWidget *Lwinadddevice = g_hash_table_lookup(Ldialoghash, "window");
+  GtkWidget *Lmessage;
+
+  while (1) {
+    if (gtk_dialog_run(GTK_DIALOG(Lwinadddevice)) != GTK_RESPONSE_CANCEL) {
+      GtkEntry *Ldevicebus = GTK_ENTRY(g_hash_table_lookup(Ldialoghash, "devicebus"));
+      GtkEntry *Ldevicename = GTK_ENTRY(g_hash_table_lookup(Ldialoghash, "devicename"));
+      gchar *Lbus = (gchar *)gtk_entry_get_text(Ldevicebus);
+      Tdriveinfo *Ldrive = g_malloc0(sizeof(Tdriveinfo));
+    
+      maj_drive_info(Ldrive, Lbus, NULL);
+
+      if (Ldrive->type == 0) {
+        Lmessage = gtk_message_dialog_new(GTK_WINDOW(Lparent),
+                                     GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
+                                     _("Cannot add this drive !"));
+      } else {
+        Lmessage = gtk_message_dialog_new(GTK_WINDOW(Lparent),
+                                     GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_OK,
+                                     _("Drive succesfully added !"));
+      }
+      
+      gtk_dialog_run(GTK_DIALOG(Lmessage));
+      gtk_widget_destroy(Lmessage);
+
+      if (Ldrive->type > 0) {
+        /* on complate la structure drive avant de l'ajouter */
+        Ldrive->manual = TRUE;
+        Ldrive->actuperiph = 0;
+        Ldrive->name = g_strdup(gtk_entry_get_text(Ldevicename));
+        Ldrive->dev = g_slist_append(Ldrive->dev, g_strdup(Lbus));
+        Ldrive->bus = g_slist_append(Ldrive->bus, g_strdup("MANU"));
+
+        Glistdrives = g_slist_append(Glistdrives, Ldrive);
+
+        /* mise a jour de la liste des peripheriques */
+        remplirelistegraveur(Lliste);
+        break;
+      }
+      g_free(Ldrive);
+    } else {
+      break;
+    }
+  }
+  gtk_widget_destroy(Lwinadddevice);
+  g_hash_table_destroy(Ldialoghash);
+}
+
+  
 /* selection d'une image iso */
 void gtk_file_sel(GtkWidget *Abtn, gpointer Adata)
 {
@@ -623,12 +755,15 @@ void on_selectgraveur(GtkWidget *Acombo, gpointer Adata)
   gchar *Llecteur = NULL;
   guint Lident = 0;
   gint i;
+  Tdriveinfo *Ldrive;
 
   if (g_str_has_suffix(Lname, "combo")==FALSE) return;
         
   if (!gtk_combo_box_get_active_iter(GTK_COMBO_BOX(Acombo), &Liter)) return;
   gtk_tree_model_get(Lmodel, &Liter, 0, &Llecteur, 3, &Lident, -1);
 
+  Ldrive = (Tdriveinfo *) g_slist_nth_data(Glistdrives, Lident);
+
   Lisimgiso = !strcmp(Llecteur, "iso"); 
 
   Laskname = g_strdup(Lname);
@@ -666,20 +801,22 @@ void on_selectgraveur(GtkWidget *Acombo, gpointer Adata)
     /* disable si image iso, enable sinon */
     gtk_widget_set_sensitive (Lobj, !Lisimgiso);
   }
+
   for (i=0; ((Lobj = sc_hash_table_lookupv(Lhash, "%s_notcdrwtodisabled%d", Lname, i)) && GTK_IS_WIDGET(Lobj)); i++) {
     /* disable si lecteur non cdrw */
-    gtk_widget_set_sensitive (Lobj, DRIVE_CDRW_WRITER(Lident));
+    gtk_widget_set_sensitive (Lobj, DRIVE_WRITER(Ldrive));
   }
 
   g_free(Llecteur);
 }
 
 /* confirmer l'annulation de la gravure */
-void abort_gravure(GtkWidget *Abtn, gpointer Adata)
+gboolean abort_gravure(GtkWidget *Abtn, gpointer Adata)
 {
   GHashTable *Lhash = (GHashTable *) Adata;
   gboolean *Labort = (gboolean *)g_hash_table_lookup(Lhash, "gabort");
   GtkWidget *Lwindow = GTK_WIDGET(g_hash_table_lookup(Lhash, "window_burn"));
+  gint Lret;
 
   g_assert(Labort);
 
@@ -687,7 +824,9 @@ void abort_gravure(GtkWidget *Abtn, gpointer Adata)
                       GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION,
                       GTK_BUTTONS_YES_NO, _("Do you really want to abort operation ?"));
 
-  if (gtk_dialog_run(GTK_DIALOG(Lconfirm)) == GTK_RESPONSE_YES) {
+  Lret = gtk_dialog_run(GTK_DIALOG(Lconfirm));
+      
+  if (Lret == GTK_RESPONSE_YES) {
     gint *Lpid = (gint *)g_hash_table_lookup(Lhash, "pid");
     if (*Lpid) {
       /* on envoi un TERM au process (cdrecord...) pour qu'il s'arrete ! */
@@ -697,6 +836,16 @@ void abort_gravure(GtkWidget *Abtn, gpointer Adata)
   }
 
   gtk_widget_destroy(Lconfirm);
+
+  return Lret == GTK_RESPONSE_YES ? TRUE : FALSE;
+}
+
+gboolean on_deleteevent_gravure(GtkWidget *Awidget, GdkEvent *Aevent, gpointer Adata)
+{
+
+  abort_gravure(NULL, Adata);
+
+  return TRUE;
 }
 
 /* confirmer la cloture d'un cd */
@@ -824,13 +973,34 @@ void view_popupmenu_addfile(GtkWidget *Aliste, GdkEventButton *Aevent, gpointer
   GtkMenu *Lpopupm = create_gtkpopupmenudata(Aliste, Adata);
   gtk_menu_popup(Lpopupm, NULL, NULL, NULL, Aliste, Aevent->button, Aevent->time);
 }
+void key_popupmenu_addfile(GtkWidget *Aliste, gpointer Adata) {
+  GtkMenu *Lpopupm = create_gtkpopupmenudata(Aliste, Adata);
+  gtk_menu_popup(Lpopupm, NULL, NULL, NULL, Aliste, 0, 0);
+}
+
 
 /* popup ajouter des pistes audio */
 void view_popupmenu_addaudio(GtkWidget *Aliste, GdkEventButton *Aevent, gpointer Adata) {
   GtkMenu *Lpopupm = create_gtkpopupmenuaudio(Aliste, Adata);
   gtk_menu_popup(Lpopupm, NULL, NULL, NULL, Aliste, Aevent->button, Aevent->time);
 }
+void key_popupmenu_addaudio(GtkWidget *Aliste, gpointer Adata) {
+  GtkMenu *Lpopupm = create_gtkpopupmenuaudio(Aliste, Adata);
+  gtk_menu_popup(Lpopupm, NULL, NULL, NULL, Aliste, 0, 0);
+}
 
+
+/* popup gestion des peripheriques */
+void view_popupmenu_devices(GtkWidget *Aliste, GdkEventButton *Aevent, gpointer Adata) {
+  GtkMenu *Lpopupm = create_gtkpopupmenudevices(Aliste, Adata);
+  gtk_menu_popup(Lpopupm, NULL, NULL, NULL, Aliste, Aevent->button, Aevent->time);
+}
+void key_popupmenu_devices(GtkWidget *Aliste, gpointer Adata) {
+  GtkMenu *Lpopupm = create_gtkpopupmenudevices(Aliste, Adata);
+  gtk_menu_popup(Lpopupm, NULL, NULL, NULL, Aliste, 0, 0);
+}
+
+/* clique sur la souris, gestion des popups */
 gboolean gtk_do_eventbuttonpress(GtkWidget *Aliste, GdkEventButton *Aevent, gpointer Adata)
 {
   if (!Aevent) return FALSE;
@@ -842,6 +1012,17 @@ gboolean gtk_do_eventbuttonpress(GtkWidget *Aliste, GdkEventButton *Aevent, gpoi
         view_popupmenu_addfile(Aliste, Aevent, Adata);
       } else if (!strcmp(Lname, "LISTEAUDIO")) {
         view_popupmenu_addaudio(Aliste, Aevent, Adata);
+      } else if (!strcmp(Lname, "periphdata")) {
+        view_popupmenu_devices(Aliste, Aevent, Adata);
+      }
+      return TRUE;
+    }
+  } else if (Aevent->type == GDK_2BUTTON_PRESS) {
+    if (Aevent->button == 1) {
+      const gchar *Lname = glade_get_widget_name(Aliste);
+      /* double clique sur la fenetre des lecteurs, on edite le lecteur */
+      if (!strcmp(Lname, "periphdata")) {
+        gtk_devices_edit(NULL, Adata);
       }
       return TRUE;
     }
@@ -985,14 +1166,9 @@ void menu_active_proprietes(GtkWidget *Amenuitem, gpointer Adata)
   TProgRequis *Lcurentry;
   gchar *Lconffile;
   gchar Ltmp[256];
-/*  GtkComboBox *Ldstaudiocombo = GTK_COMBO_BOX(g_hash_table_lookup(Lhash, "dstaudiocombo"));
-  GtkComboBox *Ldstdatacombo = GTK_COMBO_BOX(g_hash_table_lookup(Lhash, "dstdatacombo"));
-  GtkComboBox *Lsrccopiecombo = GTK_COMBO_BOX(g_hash_table_lookup(Lhash, "srccopycombo"));
-  GtkComboBox *Ldstcopiecombo = GTK_COMBO_BOX(g_hash_table_lookup(Lhash, "dstcopycombo"));*/
 
   gtk_dialog_run(GTK_DIALOG(Lwinproprietes));
 
-
   /* recuperation des valeurs a sauver */
   Lptr = g_hash_table_lookup(Ldialoghash, "dirtmpvalue");
   conf_store_value("tmpdir", (gchar *)gtk_entry_get_text(GTK_ENTRY(Lptr)));
@@ -1081,9 +1257,10 @@ void cherchelecteur(GtkWidget *Aobj, gpointer Adata)
   GtkWidget *Lcancel;
   gboolean Labort = FALSE;
   gint Lcont = 2;
-  gint i;
   gchar *Ltxt;
   GHashTable *Lhash = _hash();
+  GSList *Lcur;
+  Tdriveinfo *Ldrive;
   
   Lwin = glade_xml_get_widget(Lxmlencours, "window_inprogress"); 
 
@@ -1101,13 +1278,19 @@ void cherchelecteur(GtkWidget *Aobj, gpointer Adata)
   gtk_widget_show(Lwin);
   g_timeout_add(500, encours_callback, Lhash);
 
-  scan_for_drives(NULL);
+  scan_for_drives(Lhash, NULL);
 
-  for (i=0; i < Gnumdrives; i++) {
-    Ltxt = g_strdup_printf(_("Scanning drive \"%s\"..."), Gldrives[i].name);
-    gtk_label_set_text(GTK_LABEL(Llabel), Ltxt); g_free(Ltxt);
-    while(gtk_events_pending()) gtk_main_iteration();
-    maj_drive_info(i, NULL);
+  if (Labort == FALSE) {
+    for (Lcur = Glistdrives; Lcur; Lcur = g_slist_next(Lcur)) {
+      Ldrive = (Tdriveinfo *)Lcur->data;
+      if (g_slist_length(Ldrive->dev)>0) {
+        Ltxt = g_strdup_printf(_("Scanning drive \"%s\"..."), Ldrive->name);
+        gtk_label_set_text(GTK_LABEL(Llabel), Ltxt); g_free(Ltxt);
+        while(gtk_events_pending()) gtk_main_iteration();
+
+        maj_drive_info(Ldrive, (gchar *)g_slist_nth_data(Ldrive->dev, 0), NULL);
+      }
+    }
   }
 
   /* on attend la fin d'execution de la popup d'attente */
@@ -1116,7 +1299,7 @@ void cherchelecteur(GtkWidget *Aobj, gpointer Adata)
 
   gtk_widget_destroy(Lwin);
   g_hash_table_destroy(Lhash);
-  if (GTK_IS_WIDGET(Aobj)) {
+  if (GTK_IS_WIDGET(Aobj) && Labort == FALSE) {
     Ldata = GTK_TREE_VIEW(g_hash_table_lookup(Lparenthash, "periphdata"));
     remplirelistegraveur(Ldata);
 
