@@ -18,7 +18,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
  * MA 02111-1307, USA. 
  * 
- * URL: http://scresto.site.voite.fr/
+ * URL: http://www.nongnu.org/graveman/
  *
  */
 
@@ -58,17 +58,19 @@
 #define CDRECORD_INCOMPATIBLE_MEDIUM "cannot format medium - incompatible medium"
 #define CDRECORD_FIXATING_TIME "Fixating time:"
 
-typedef struct {
-  char *cmdline;
-  char *bus;
-} Tsearch_drive;
 
 /* definition de tous les type de bus */
-Tsearch_drive listesearchdrives[] = {
-    { "", " " }, /* scsi (sg) */
-    { "dev=ATA", "ATA:" }, /* ide sg */
-    { "dev=ATAPI", "ATAPI:" }, /* ide */
-    { NULL, NULL}
+Tsearchdrive listesearchdrives[] = {
+    { "", "", " ", 100 }, /* scsi (sg) */
+    { "dev=ATA:", "ATA:", "ATA", 80 }, /* ide sg */
+    { "dev=ATAPI:", "ATAPI:", "ATAPI", 60 }, /* ide */
+#ifdef LINUX_IDE  /* pure ide devices with linux */
+    { "dev=/dev/hda", "/dev/hda", "IDE", 40 },
+    { "dev=/dev/hdb", "/dev/hdb", "IDE", 40 },
+    { "dev=/dev/hdc", "/dev/hdc", "IDE", 40},
+    { "dev=/dev/hdd", "/dev/hdd", "IDE", 40 },
+#endif
+    { NULL, NULL, NULL, 0}
   };
 
 gboolean cdrecord_grave_callback(GIOChannel *Astd, GIOCondition Acond, gpointer Adata);
@@ -130,29 +132,35 @@ gboolean extractlecteur(gchar *Achaine, gchar **Adev, gchar **Amarque, gchar **A
 gboolean scan_callback(GIOChannel *Astd, GIOCondition Acond, gpointer Adata)
 {
   GIOStatus Lstatus;
+  GHashTable *Lhash = (GHashTable *)Adata;
+  gint *Lcont = (gint *)g_hash_table_lookup(Lhash, "cont");
+  Tsearchdrive *Ldrivedesc = (Tsearchdrive *)g_hash_table_lookup(Lhash, "drivedesc");
   gchar *Lbuffer = NULL;
-  gchar *Lbustype = (gchar *)Adata;
   gchar *Lmodel, *Lmarque, *Ldev, *Lquoi;
   
   /* fin du callback lorsque l'on recoi un signal comme quoi le pipe est ferme */
   if (Acond == G_IO_HUP || Acond == G_IO_ERR) {
-    *Lbustype = 0;
+    *Lcont = 2;
     return FALSE;
   }
   
   Lstatus = g_io_channel_read_line(Astd, &Lbuffer, NULL, NULL, NULL);  
-  if (!Lbuffer) return FALSE;
+  if (!Lbuffer) { *Lcont = FALSE; return FALSE; }
     
   g_strstrip(Lbuffer);  
-  
-  if (extractlecteur(Lbuffer, &Ldev, &Lmarque, &Lmodel, &Lquoi) == TRUE && !is_duplicate_drives(Lbustype, Ldev)) {
-    
+/* _DEB("scan lecteur = %s\n", Lbuffer); */
+  if (extractlecteur(Lbuffer, &Ldev, &Lmarque, &Lmodel, &Lquoi) == TRUE) {
+
+    insert_or_update_drive(Ldev, Lmarque, Lmodel, Lquoi, Ldrivedesc);
+
+    _DEB("oui un lecteur");
+/*    
     g_snprintf(Gldrives[Gnumdrives].dev, sizeof(Gldrives[Gnumdrives].dev)-1, "%s%s", Lbustype, Ldev);
     g_snprintf(Gldrives[Gnumdrives].name, sizeof(Gldrives[Gnumdrives].name)-1,
       "%s %s %s", Lmarque, Lmodel, Lquoi);
     Gldrives[Gnumdrives].type = -1;
     Gldrives[Gnumdrives].vitesse = 0;
-    Gnumdrives++;
+    Gnumdrives++;*/
     
     g_free(Lquoi);
     g_free(Lmodel);
@@ -166,28 +174,30 @@ gboolean scan_callback(GIOChannel *Astd, GIOCondition Acond, gpointer Adata)
 }
 
 /* scan des lecteurs */
-gboolean scan_for_drives(GError **Aerror)
+gboolean scan_for_drives(GHashTable *Ahash, GError **Aerror)
 {
   gchar **Lcmd;
   gchar *Lcommandline;
   gint pid, g_out, Lnbrarg;
   gboolean Lbolstatus;
   GIOChannel *Lcom;
-  GIOStatus Lstatus;
+  gboolean Lstatus;
   guint Lcomevent;
-  gchar *Lbustype;
-  Tsearch_drive *Lcurentry;
+  Tsearchdrive *Lcurentry;
+  gint *Lcont = (gint *)g_hash_table_lookup(Ahash, "cont");
+  gboolean *Labort = (gboolean *)g_hash_table_lookup(Ahash, "gabort");
   
-  Gnumdrives=0;
+  free_drives(FALSE);
 
-  for (Lcurentry = listesearchdrives; Lcurentry->cmdline; Lcurentry++) {
-    Lcommandline = g_strdup_printf("%s -scanbus %s", conf_get_string("cdrecord"), Lcurentry->cmdline);
+  for (Lcurentry = listesearchdrives; Lcurentry->detectline && *Labort == FALSE; Lcurentry++) {
+    Lcommandline = g_strdup_printf("%s -scanbus %s", conf_get_string("cdrecord"), Lcurentry->detectline);
   _DEB("commande =%s\n", Lcommandline);
     Lstatus = g_shell_parse_argv(Lcommandline, &Lnbrarg, &Lcmd, Aerror);
     g_free(Lcommandline);
-    if (Lstatus == FALSE) {
-      return FALSE;
-    }
+    if (Lstatus == FALSE) return FALSE;
+
+    *Lcont = 3;
+    g_hash_table_insert(Ahash, "drivedesc", Lcurentry);
 
     Lbolstatus = g_spawn_async_with_pipes(NULL, Lcmd, NULL, /* env argument */
       (GSpawnFlags ) (0),
@@ -199,31 +209,28 @@ gboolean scan_for_drives(GError **Aerror)
       return FALSE;
     }
 
-    Lbustype = g_strdup(Lcurentry->bus);
-
     Lcom = g_io_channel_unix_new( g_out );
     g_io_channel_set_encoding (Lcom, NULL, NULL);
     g_io_channel_set_flags( Lcom, G_IO_FLAG_NONBLOCK, NULL );
     Lcomevent = g_io_add_watch (Lcom, (G_IO_IN | G_IO_HUP | G_IO_ERR | G_IO_PRI),
   					     scan_callback,
-					     Lbustype);
-  
-    while (*Lbustype) {
+					     Ahash); 
+    while (*Lcont==3) {
       gtk_main_iteration();    
     }
-    
+
     g_source_remove(Lcomevent);
       
-    g_free(Lbustype);
     g_io_channel_shutdown(Lcom, FALSE, NULL);
     g_io_channel_unref(Lcom);
     g_spawn_close_pid(pid);
+    g_hash_table_remove(Ahash, "drivedesc");
   }
 
   return TRUE;
 }
 
-gboolean maj_drive_info(gint Anumdrive, GError **Aerror)
+gboolean maj_drive_info(Tdriveinfo *Adrive, gchar *Adev, GError **Aerror)
 {
   gchar *Lcommandline, *Lout = NULL, *Lerr = NULL;
   gchar *s = NULL, *f;
@@ -231,7 +238,8 @@ gboolean maj_drive_info(gint Anumdrive, GError **Aerror)
   gboolean Lstatus;
   gint Ltype = 0;
 
-  Lcommandline = g_strdup_printf("%s -prcap dev=%s", conf_get_string("cdrecord"), Gldrives[Anumdrive].dev);
+  Lcommandline = g_strdup_printf("%s -prcap dev=%s", conf_get_string("cdrecord"), Adev);
+_DEB("==================%s\n", Lcommandline);  
   Lstatus = g_spawn_command_line_sync(Lcommandline, &Lout, &Lerr, &Lexit, Aerror);
   g_free(Lcommandline);
   if (Lstatus == FALSE || !Lout || !Lerr || !*Lout) {
@@ -261,10 +269,10 @@ gboolean maj_drive_info(gint Anumdrive, GError **Aerror)
 
   if (!s) return FALSE;
 
-  Gldrives[Anumdrive].type = Ltype; /* type: lecteur ou graveur */
-  Gldrives[Anumdrive].vitesse = atoi(s);  /* vitesse de lecture ou de gravure maximum */
-
-  _DEB("donc la vitesse de lecture => %d\n", Gldrives[Anumdrive].vitesse  );
+  Adrive->type = Ltype; /* type: lecteur ou graveur */
+  Adrive->vitesse = atoi(s);  /* vitesse de lecture ou de gravure maximum */
+  _DEB("== le type => %d\n", Ltype);
+  _DEB("== donc la vitesse de lecture => %d\n", Adrive->vitesse  );
   return TRUE;
 }
 
@@ -838,6 +846,11 @@ _DEB("===>%s", Lbuffer);
     GError **Lerreur = (GError **) g_hash_table_lookup(Lhash, "gerror"); /* pointeur erreur */
     if (!*Lerreur)
       g_set_error(Lerreur, GRAVEMAN_ERROR, _ERR_INCOMPATIBLE_MEDIUM, _("Cannot blank disk, this is not a CD-RW !"));
+  } else if (strstr(Lbuffer, CDRECORD_ERRORDISK)) {
+    /* erreur pas de cd */
+    GError **Lerreur = (GError **) g_hash_table_lookup(Lhash, "gerror"); /* pointeur erreur */
+    if (!*Lerreur)
+      g_set_error(Lerreur, GRAVEMAN_ERROR, _ERR_NO_CD, _("Error, a CD-R/CD-RW is required in the cd recorder !"));
   }
 
   g_free(Lbuffer);
