@@ -97,14 +97,31 @@ void foreachmedia(gpointer Amedia, gpointer Adata)
   fprintf(Lfic, "audiolabel=%s\n", Amediaentry->labelaudio);
 }
 
+/* callback ecriture d'un lecteur */
+void foreachdrives(gpointer Adrive, gpointer Adata)
+{
+  FILE *Lfic = (FILE *) Adata;
+  Tdriveinfo *Ldrive = (Tdriveinfo *) Adrive; 
+  gint i;
+
+  fprintf(Lfic, "[lecteur]\n");
+  for (i=0; i < g_slist_length(Ldrive->bus); i++) {
+    fprintf(Lfic, "dev=%s|%s\n", (gchar *)g_slist_nth_data(Ldrive->bus, i),(gchar *)g_slist_nth_data(Ldrive->dev, i));
+  }
+  fprintf(Lfic, "actu=%d\n", Ldrive->actuperiph);  /* numero du dev utilise */
+  fprintf(Lfic, "name=%s\n", Ldrive->name);  
+  fprintf(Lfic, "type=%d\n", Ldrive->type);
+  fprintf(Lfic, "vitesse=%d\n", Ldrive->vitesse);
+  fprintf(Lfic, "manual=%d\n", Ldrive->manual == TRUE ? 1 : 0);
+}
+
 /* ecriture du fichier de configuration */
 gint write_conf(gchar *AFichier, GError **Aerror)
 {
-  FILE *fic;
+  FILE *Lfic;
   gchar *Ltmp;
-  gint i;
 
-  if (!(fic=fopen(AFichier, "w"))) {
+  if (!(Lfic=fopen(AFichier, "w"))) {
     Ltmp = g_strdup_printf(_("Cannot create %s: %s"), AFichier, g_strerror(errno));
     g_set_error(Aerror, G_FILE_ERROR, g_file_error_from_errno(errno), Ltmp, g_strerror(errno)); 
     g_warning("%s", Ltmp);
@@ -113,21 +130,16 @@ gint write_conf(gchar *AFichier, GError **Aerror)
     return EXIT_FAILURE;
   }
 
-  fprintf(fic, "[general]\n");
-  g_hash_table_foreach(Gconfigprog, foreachconfig, (gpointer *)(fic));
+  fprintf(Lfic, "[general]\n");
+  g_hash_table_foreach(Gconfigprog, foreachconfig, (gpointer *)(Lfic));
 
-  g_slist_foreach(Gmedialist, foreachmedia, (gpointer *)(fic));
+/*  g_slist_foreach(Gmedialist, foreachmedia, (gpointer *)(fic)); */
   
   /* ecriture des lecteurs */
-  for (i=0; i < Gnumdrives; i++) {
-    fprintf(fic, "[lecteur]\n");
-    fprintf(fic, "dev=%s\n", Gldrives[i].dev);
-    fprintf(fic, "name=%s\n", Gldrives[i].name);
-    fprintf(fic, "type=%d\n", Gldrives[i].type);
-    fprintf(fic, "vitesse=%d\n", Gldrives[i].vitesse);
-  }
 
-  fclose(fic);
+  g_slist_foreach(Glistdrives, foreachdrives, (gpointer *)(Lfic));
+
+  fclose(Lfic);
 
   return EXIT_SUCCESS;
 }
@@ -408,10 +420,8 @@ gboolean read_config(GError **Aerror)
   gulong Lmedialen = 0;
   gchar Lmedialabeldata[50];
   gchar Lmedialabelaudio[50];
+  Tdriveinfo *Ldriveinfo = NULL;
 
-  Gnumdrives = 0;
-  INIT_GDRIVES(0);
-    
   if (!g_file_get_contents(Gfileconf, &Lcontents, NULL, Aerror)) {
     g_warning("%s", (*Aerror)->message);
     return FALSE;
@@ -426,9 +436,17 @@ gboolean read_config(GError **Aerror)
     if (!*Luneligne || *Luneligne == ';' || *Luneligne == '#') continue;
 
     if (*Luneligne == '[') {
+      if (Ldriveinfo) { /* ajout d'un lecteur si un en cours */
+        append_this_drive(Ldriveinfo);
+        Ldriveinfo = NULL;
+      }
+      
       g_strlcpy(Lemplacement, Luneligne, sizeof(Lemplacement));
-      if (!strcmp(Lemplacement, "[media]")) {
+      
+      if (!strcmp(Lemplacement, "[media]")) { /* init media */
         Lmedialen = 0; *Lmedialabeldata = *Lmedialabelaudio = 0;
+      } else if (!strcmp(Lemplacement, "[lecteur]")) {  /* init lecteur */
+        Ldriveinfo = g_malloc0(sizeof(Tdriveinfo));
       }
     } else if (!strcmp(Lemplacement, "[general]")) {
       /* section general */
@@ -462,22 +480,33 @@ gboolean read_config(GError **Aerror)
       Lkey = Luneligne;
      
       if (!strcmp(Lkey, "type")) {
-        Gldrives[Gnumdrives].type = atoi(Lvalue);
+        Ldriveinfo->type = atoi(Lvalue);
       } else if (!strcmp(Lkey, "name")) {
-        g_strlcpy(Gldrives[Gnumdrives].name, Lvalue, sizeof(Gldrives[Gnumdrives].name)-1);
+        if (Ldriveinfo->name) g_free(Ldriveinfo->name);
+        Ldriveinfo->name = g_strdup(Lvalue);
       } else if (!strcmp(Lkey, "dev")) {
-        g_strlcpy(Gldrives[Gnumdrives].dev, Lvalue, sizeof(Gldrives[Gnumdrives].dev)-1);
+        gchar *Lvalue2 = strchr(Lvalue, '|');
+        if (Lvalue2) *(Lvalue2++)=0;
+        Ldriveinfo->bus = g_slist_append(Ldriveinfo->bus, Lvalue2 ? g_strdup(Lvalue) : "OLD");
+        Ldriveinfo->dev = g_slist_append(Ldriveinfo->dev, g_strdup(Lvalue2 ? Lvalue2 : Lvalue));
+      } else if (!strcmp(Lkey, "actu")) {
+        Ldriveinfo->actuperiph = atoi(Lvalue);
       } else if (!strcmp(Lkey, "vitesse")) {
-        Gldrives[Gnumdrives].vitesse = atoi(Lvalue);
-      }
-      if (Gldrives[Gnumdrives].type > -1 && *Gldrives[Gnumdrives].dev &&
-          *Gldrives[Gnumdrives].name && Gldrives[Gnumdrives].vitesse > -1) {
-        ++Gnumdrives;
-        INIT_GDRIVES(Gnumdrives);
+        Ldriveinfo->vitesse = atoi(Lvalue);
+      } else if (!strcmp(Lkey, "manual")) {
+        Ldriveinfo->manual = *Lvalue == '1' ? TRUE : FALSE;
       }
     }
   }
 
+  if (Ldriveinfo) {
+    if (g_slist_length(Ldriveinfo->dev) && Ldriveinfo->name && Ldriveinfo->vitesse) {
+      append_this_drive(Ldriveinfo);
+    } else {
+      free_this_drive(Ldriveinfo);
+    }
+  }
+
   g_strfreev(Llignes);
   g_free(Lcontents);
   clean_config();
