@@ -95,6 +95,19 @@ gchar *get_combo_value(GtkWidget *Acombo)
   return Lvalue;
 }
 
+gint get_combo_intvalue(GtkWidget *Acombo)
+{
+  GtkTreeModel *Lmodel = gtk_combo_box_get_model(GTK_COMBO_BOX(Acombo));
+  GtkTreeIter Liter;
+  gint Lvalue = -1;
+  if (!gtk_combo_box_get_active_iter(GTK_COMBO_BOX(Acombo), &Liter)) return -1;
+
+  gtk_tree_model_get(Lmodel, &Liter, 0, &Lvalue, -1);
+
+  return Lvalue;
+}
+
+
 gint make_int(gchar *Abuf)
 {
   return *Abuf + (Abuf[1] << 8) + (Abuf[2] << 16) + (Abuf[3] << 24);
